local core = CalendarExtras

local module = core.NewModule("Invite")

 --[[ ------- Locals -------- ]]--

local attendeesCache = {}
local title, button
local text = "Invite current %s to the event"

local inviteRaidContextMenu

 --[[ ------ Inviting ------ ]]--

local function CacheAttendees()
	local i = 1
	while true do
		local name = CalendarEventGetInvite(i)
		if not name then break end
		attendeesCache[name] = true
		i = i + 1
	end
end

local function InGroup()
	if GetNumRaidMembers() > 0 then
		return nil, "raid"
	elseif GetNumPartyMembers() > 0 then
		return nil, "party"
	else
		return true, "raid"
	end
end


local function InviteGroupies(self, button)
	if CalendarCanSendInvite() then
		CacheAttendees()
		local num, func
		if GetNumRaidMembers() > 0 then
			for i = 1, GetNumRaidMembers() do
				local name = GetRaidRosterInfo(i)
				if not attendeesCache[name] and not name:find("-") then CalendarEventInvite(name) end
			end
		elseif GetNumPartyMembers() > 0 then
			for i = 1, GetNumPartyMembers() do
				local name = UnitName("party"..i)
				if not attendeesCache[name] and not name:find("-") then CalendarEventInvite(name) end
			end
		end
		wipe(attendeesCache)
	end
end

local function OnShowHook(self)
	local disable, group = InGroup()
	if disable then button:Disable() else button:Enable() end
	title:SetText(text:format(group)..":")

	module:RegisterEvent("PARTY_MEMBERS_CHANGED", OnShowHook)
	module:RegisterEvent("RAID_ROSTER_UPDATE", OnShowHook)
end

local function OnHideHook(self)
	module:UnregisterEvent("PARTY_MEMBERS_CHANGED")
	module:UnregisterEvent("RAID_ROSTER_UPDATE")
end

 --[[ ---- Context Menu ---- ]]--

do
	inviteRaidContextMenu = CreateFrame("frame", "CalendarinviteRaidContextMenu", CalendarCreateEventInviteButton, "UIDropDownMenuTemplate")
	inviteRaidContextMenu:Hide()
	local menu = {
		[1] = {
			func = InviteGroupies,
		},
		[2] = {
			text = "Close",
			func = function() inviteRaidContextMenu:Hide() end,
		},
	}
	function module:CalendarCreateEventInviteButtonOnClick(frame, button, ...)
		if button == "LeftButton" then
			self.hooks[frame].OnClick(frame, button, ...)
		else
			local disable, group = InGroup()
			menu[1].text = text:format(group)
			menu[1].disabled = disable
			EasyMenu(menu, inviteRaidContextMenu, "cursor", nil, nil, "MENU")
		end
	end
end

 --[[ ---------------------- ]]--

function module.OnInitialize()
	CalendarMassInviteFrame:SetHeight(300)

	local divider = CalendarMassInviteFrame:CreateTexture("CalendarExtrasExtraDivider", "BORDER")
	divider:SetTexture([[Interface\DialogFrame\UI-DialogBox-Divider]])
	divider:SetWidth(303)
	divider:SetHeight(16)
	divider:SetPoint("BOTTOMLEFT", 10, 65)
	divider:SetTexCoord(0, 0.75390625, 0, 0.5)

	title = CalendarMassInviteFrame:CreateFontString("CalendarExtrasInviteCurrentRaidTitle", "BACKGROUND", "GameFontHighlightSmall")
	title:SetWidth(280)
	title:SetPoint("TOP", divider, "BOTTOM", 0, -4)
	title:SetJustifyH("CENTER")

	button = CreateFrame("button", "CalendarExtrasInviteCurrentRaidButton", CalendarMassInviteFrame, "CalendarEventButtonTemplate")
	button:SetText("Invite")
	button:SetPoint("BOTTOM", 0, 19)
	button:SetScript("OnClick", InviteGroupies)

	module:HookScript(CalendarMassInviteFrame, "OnShow", OnShowHook)
	module:HookScript(CalendarMassInviteFrame, "OnHide", OnHideHook)

	module:RawHookScript(CalendarCreateEventInviteButton, "OnClick", "CalendarCreateEventInviteButtonOnClick", true)
	CalendarCreateEventInviteButton:RegisterForClicks("LeftButtonUp", "RightButtonUp")
end