local core = CalendarExtras

local module = core.NewModule("Status Online")

 --[[ -------- Locals ------- ]]--

local listIsDirty

local assumeMissingAreOnline -- If true, treat players not found in guild/friend list as being online

local sortersModule

local meta = {__newindex = function(self, key, value)
	if self[key] ~= value then listIsDirty = true end
	rawset(self, key, value)
end}
local listFriends = setmetatable({}, meta)
local listGuild = setmetatable({}, meta)
--local listChannels = setmetatable({}, meta)

 --[[ ----------------------- ]]--

local buttons = {}
local function DisableButton(button)
		local buttonName = button:GetName()
		local name = _G[buttonName.."Name"]
		buttons[1], buttons[2], buttons[3] = _G[buttonName.."Class"], _G[buttonName.."Status"], sortersModule and _G[buttonName.."Role"] or nil
		name:SetTextColor(0.6, 0.6, 0.6, 0.65)
		local r, g, b
		for _, button in ipairs(buttons) do
			r, g, b = button:GetTextColor()
			button:SetTextColor(r, g, b, 0.5)
		end
end

local function IsOnline(name)
	return listFriends[name] or listGuild[name] or (UnitInRaid(name) and UnitIsConnected(name)) or (UnitInParty(name) and UnitIsConnected(name)) --[[ or listChannels[name] ]]
end

function module:FRIENDLIST_UPDATE()
	for i = 1, GetNumFriends() do
		local name, _, _, _, online = GetFriendInfo(i)
		if (assumeMissingAreOnline and not online) or (not assumeMissingAreOnline and online) then
			listFriends[name] = true
		else
			listFriends[name] = nil
		end
	end
	if listIsDirty then
		listIsDirty = nil
		core.UpdateOpenList()
	end
end

function module:GUILD_ROSTER_UPDATE()
	for i = 1, GetNumGuildMembers(true) do
		local name, _, _, _, _, _, _, _, online = GetGuildRosterInfo(i)
		if (assumeMissingAreOnline and not online) or (not assumeMissingAreOnline and online) then
			listGuild[name] = true
		else
			listGuild[name] = nil
		end
	end
	if listIsDirty then
		listIsDirty = nil
		core.UpdateOpenList()
	end
end

local function UpdateInviteListButtons(_, frame)
	for _, button in ipairs(frame.buttons) do
		local inviteIndex = button.inviteIndex
		if inviteIndex then
			local name = CalendarEventGetInvite(inviteIndex)
			local found = IsOnline(name)
			if (assumeMissingAreOnline and found) or (not assumeMissingAreOnline and not found) then
				DisableButton(button)
			end
		end
	end
end

function module.OnInitialize()
	if core.modules["Sorters"] then
		sortersModule = core.modules["Sorters"]
	end

	core.RegisterCallback(module, "Post_Update", UpdateInviteListButtons)

	core.RegisterCallback(module, "CalendarShow", function()
		module:RegisterEvent("GUILD_ROSTER_UPDATE")
		module:RegisterEvent("FRIENDLIST_UPDATE")
		module:GUILD_ROSTER_UPDATE()
		module:FRIENDLIST_UPDATE()
	end)       
	core.RegisterCallback(module, "CalendarHide", function()
		module:UnregisterEvent("GUILD_ROSTER_UPDATE")
		module:UnregisterEvent("FRIENDLIST_UPDATE")
	end)
	module:RegisterEvent("GUILD_ROSTER_UPDATE")
	module:RegisterEvent("FRIENDLIST_UPDATE")
	module:GUILD_ROSTER_UPDATE()
	module:FRIENDLIST_UPDATE()
	GuildRoster()
end

--[[
local channel, header, collapsed, channelNumber, count, active, category = GetChannelDisplayInfo(id)
local name, owner, moderator, muted, active, enabled = GetChannelRosterInfo(id, rosterIndex)
SetSelectedDisplayChannel(id)
CHANNEL_ROSTER_UPDATE(channelId)
]]